import axios, {AxiosError} from 'axios'
import {onMounted, reactive, Ref, ref} from 'vue'
import AuthService, {ILoginData, IRegistrationData, IUser, IUserData} from '@/services/authService'
import $api, {axiosConfig} from '@/http/index'
import router from '@/router/index'

export interface IStore extends Omit<ReturnType<typeof useStore>['store'], 'user'> {
  user: Ref<IUser>
}

export function useStore() {
  const isAuth = ref(false)
  const user = reactive<{ value: Partial<IUser> }>({value: {}})
  const isAllowRegistration = ref(false)
  const loaded = ref(false)

  async function registration(authData: IRegistrationData) {
    if (isAllowRegistration.value) {
      try {
        const response = await AuthService.registration(authData)
        localStorage.setItem('token', response.data.accessToken)
        isAuth.value = true
        user.value = response.data.user
        setMetricaUser(response.data.user)
        await router.replace('/profile')
        return null
      } catch (e) {
        const message = (<AxiosError>e).response?.data.message
        // console.error('[err] in registration, message: ', message)
        return message
      }
    }
  }

  async function login(authData: ILoginData) {
    try {
      const response = await AuthService.login(authData)
      localStorage.setItem('token', response.data.accessToken)
      isAuth.value = true
      user.value = response.data.user
      setMetricaUser(response.data.user)
      window.scrollTo({top: 0, behavior: 'smooth'})
      await router.replace('/profile')
      return null
    } catch (e) {
      const message = (<AxiosError>e).response?.data?.message
      // console.error('[err] in login, message: ', message)
      return message
    }
  }

  async function logout() {
    try {
      const response = await AuthService.logout()
      // console.log('[logout] ', response);
      localStorage.removeItem('token')
      isAuth.value = false
      user.value = {}
      window.scrollTo({top: 0, behavior: 'smooth'})
      await router.replace('/auth')
    } catch (e) {
      // console.error('[err] in logout, message: ', (<AxiosError>e).response?.data?.message)
    }
  }

  async function refresh() {
    try {
      const response = await axios.get<IUserData>('/refresh', axiosConfig)
      // console.log('[refresh] ', response)
      localStorage.setItem('token', response.data.accessToken)
      isAuth.value = true
      user.value = response.data.user
      setMetricaUser(response.data.user)
    } catch (e) {
      // console.error('[err] in refresh, message: ', (<AxiosError>e).response?.data?.message)
    }
  }

  async function changeRegState() {
    try {
      const response = await $api.post('/variables', {key: 'isAllowRegistration', val: !isAllowRegistration.value})
      if (response.status === 200) isAllowRegistration.value = !isAllowRegistration.value
    } catch (e) {
      console.error('[err] in changeRegState, message: ', (<AxiosError>e).response?.data?.message)
    }
  }

  onMounted(async () => {
    await refresh()
    isAllowRegistration.value = (await axios.get('/variables', {...axiosConfig, params: {key: 'isAllowRegistration'}})).data === true
    loaded.value = true
  })

  return {
    store: reactive({
      isAuth,
      user,
      registration,
      login,
      logout,
      isAllowRegistration,
      changeRegState,
    }),
    loaded,
  }
}

function setMetricaUser(user: any) {
  if (user.id < 300) {
    const ymUser = {...user}
    ymUser.UserID = ymUser.id
    delete ymUser.id
    // @ts-ignore
    ym(84425530, 'userParams', ymUser)
  }
}
