import {validationResult} from 'express-validator'
import ApiError from '../exceptions/api-error'
import userService, {ILoginData, IRegistrationData, IStatusChanges} from '../service/user-service'
import avatarService from '../service/avatar-service'
import {IRequest, Response} from 'express'

const refreshMaxAge = Number(process.env.JWT_REFRESH_LIVETIME) * 24 * 60 * 60 * 1000

class UserController {
  async registration(req: IRequest<IRegistrationData>, res: Response, next) {
    try {
      const errors = validationResult(req)
      if (!errors.isEmpty()) return next(ApiError.BadRequest('Ошибка при валидации', errors.array()))
      const authData = req.body
      const {userData, refreshToken} = await userService.registration(authData)
      res.cookie('refreshToken', refreshToken, {maxAge: refreshMaxAge, httpOnly: true, secure: true})
      return res.json(userData)
    } catch (e) {next(e)}
  }

  async login(req: IRequest<ILoginData>, res: Response, next) {
    try {
      const {email, password} = req.body
      const {userData, refreshToken} = await userService.login(email, password)
      res.cookie('refreshToken', refreshToken, {maxAge: refreshMaxAge, httpOnly: false, secure: false})
      return res.json(userData)
    } catch (e) {next(e)}
  }

  async activate(req: IRequest<{}, { link: string }>, res: Response, next) {
    try {
      const activationLink = req.params.link
      await userService.activate(activationLink)
      return res.redirect(`${process.env.CLIENT_URL}/profile`)
    } catch (e) {next(e)}
  }

  async logout(req: IRequest, res: Response, next) {
    try {
      const {refreshToken} = req.cookies
      await userService.logout(refreshToken)
      res.clearCookie('refreshToken')
      return res.json({status: 'OK'})
    } catch (e) {next(e)}
  }

  async refresh(req: IRequest, res: Response, next) {
    try {
      const {refreshToken} = req.cookies
      const {userData, newRefreshToken} = await userService.refresh(refreshToken)
      res.cookie('refreshToken', newRefreshToken, {maxAge: refreshMaxAge, httpOnly: false, secure: false})
      return res.json(userData)
    } catch (e) {next(e)}
  }

  async setAvatar(req: IRequest, res: Response, next) {
    try {
      const {user} = req
      const [img] = Object.values(req.files!)
      if (img.mimetype !== 'image/png' && img.mimetype !== 'image/jpeg') return next(ApiError.UnsupportedMediaType('Формат файла не является png или jpeg'))
      if (img.size > 5000000) return next(ApiError.PayloadTooLarge('Размер файла превышает 5мб'))
      return res.json(await avatarService.set(user.id, img))
    } catch (e) {next(e)}
  }

  async getStudents(req, res: Response, next) {
    try {
      return res.json(await userService.getStudents())
    } catch (e) {next(e)}
  }

  async getStatuses(req, res: Response, next) {
    try {
      return res.json(await userService.getStatuses())
    } catch (e) {next(e)}
  }

  async changeStatuses(req: IRequest<IStatusChanges>, res: Response, next) {
    try {
      const statusChanges = req.body
      return res.json(await userService.changeStatuses(statusChanges))
    } catch (e) {next(e)}
  }

  async changeTelegram(req: IRequest<{ telegram: string }>, res: Response, next) {
    try {
      const {user, body: {telegram}} = req
      await userService.changeTelegram(telegram, user.id)
      return res.json({status: 'OK'})
    } catch (e) {next(e)}
  }
}

export = new UserController()
