import mysql from 'mysql2/promise'
import ApiError from '../exceptions/api-error'

export interface ITask {
  id: number,
  title: string | null,
  task: string,
  members: number,
  limit: number,
  deadline: string
}

export interface IReports {
  [key: number]: {
    [key: number]: IReport
  }
}

export interface IReport {
  userId: number,
  reportId: number,
  enrolDate: string,
  surrenderDate: string | null,
  answerLink: string,
  credited: 0
}

export type ITaskSelf = ITask & Partial<IReport>

class reportService {
  async getReports() {
    const [tasks] = await mysql.db.query<ITask, '!'>('SELECT *, DATE_FORMAT(`deadline`,\'%Y-%m-%d\') as `deadline` FROM `reportTasks`')
    return tasks
  }

  async getReportProgress() {
    const [unparsedReports] = await mysql.db.query<IReport, '!'>('SELECT * FROM `reportAnswers`')
    const reports: IReports = {}
    for (const el of unparsedReports) {
      if (!reports[el.userId]) reports[el.userId] = {}
      reports[el.userId][el.reportId] = el
    }
    return reports
  }

  // async changeReportProgress(userId, reportId, value) {
  //     return await mysql.db.query('UPDATE `reportAnswers` SET `credited` = ? WHERE `userId` = ? AND `reportId` = ?', [value, userId, reportId])
  // }

  async getReportsSelf(userId: number) {
    const [reports] = await mysql.db.query<ITaskSelf>('SELECT `t`.*, DATE_FORMAT(`t`.`deadline`,\'%Y-%m-%d\') as `deadline`, `a`.* FROM `reportTasks` `t` LEFT OUTER JOIN `reportAnswers` `a` ON `t`.`id` = `a`.`reportId` AND `a`.`userId` = ?', [userId])
    return reports
  }

  async uploadReport(userId: number, reportId: number, link: string) {
    return await mysql.db.query('UPDATE `reportAnswers` SET `answerLink` = ?, `surrenderDate` = ? WHERE `userId` = ? AND `reportId` = ?', [link, Date.now(), userId, reportId])
  }

  async setReport(reportId: number | undefined, title: string | null, task: string, deadline?: string, limit?: number) {
    if (reportId) {
      await mysql.db.query('UPDATE `reportTasks` SET `title` = ?, `task` = ?, `deadline` = ?, `limit` = ? WHERE `id` = ?', [title, task, deadline, limit, reportId])
      return {id: reportId}
    } else {
      const [result] = await mysql.db.query('INSERT INTO `reportTasks` (`title`, `task`, `deadline`, `limit`) VALUES (?, ?, ?, ?)', [title, task, deadline, limit])
      return {id: result?.insertId}
    }
  }

  async getProject(reportId: number) {
    const [project] = await mysql.db.query<IReport, '!'>('SELECT * FROM `reportAnswers` WHERE `reportId` = ?', [reportId])
    return project
  }

  async enrolment(userId: number, reportId: number, value: boolean) {
    if (value) {
      const [[project]] = await mysql.db.query<Pick<ITask, 'members' | 'limit'>>('SELECT `members`, `limit` FROM `reportTasks` WHERE `id` = ?', [reportId])
      if (project && project.members < project.limit) {
        await mysql.db.query('INSERT INTO `reportAnswers` (`userId`, `reportId`, `enrolDate`) VALUES (?, ?, ?)', [userId, reportId, Date.now()])
        await mysql.db.query('UPDATE `reportTasks` SET `members` = `members` + 1 WHERE `id` = ?', [reportId])
      } else throw ApiError.Conflict('Превышен лимит на запись')
    } else {
      const isParticipates = !!((await mysql.db.query<IReport>('SELECT * FROM `reportAnswers` WHERE `userId` = ?', [userId]))[0][0])
      if (isParticipates) {
        await mysql.db.query('DELETE FROM `reportAnswers` WHERE `userId` = ? AND `reportId` = ?', [userId, reportId])
        await mysql.db.query('UPDATE `reportTasks` SET `members` = `members` - 1 WHERE `id` = ?', [reportId])
      }
    }
    return true
  }

  async delete(projectId: number) {
    await mysql.db.query('DELETE FROM `reportTasks` WHERE `id` = ?', [projectId])
  }

  // async getReport(userId, reportId) {
  //     const [answerLinks] = await mysql.db.query('SELECT `answerLink` FROM `reportAnswers` WHERE `userId` = ? AND `reportId` = ?', [userId, reportId])
  //     return answerLinks[0] ? answerLinks[0].answerLink : null
  // }
}

export default new reportService()
