import mysql from 'mysql2/promise'

import mailService from './mail-service'
import tokenService from './token-service'
import avatarService from './avatar-service'
import questionnaireService from './questionnaire-service'
import ApiError from '../exceptions/api-error'

import bcrypt from 'bcryptjs'
import {nanoid} from 'nanoid'
import UserDto, {IUserDto} from '../dtos/user-dto'

export interface ILoginData {
  email: string,
  password: string
}

export interface IRegistrationData extends ILoginData {
  name: string,
  surname: string,
  tel: string,
  telegram: string,
  policy: boolean
}

export type IStatusChanges = {
  email: string,
  status: number
}[]

class userService {
  async registration({email, password, name, surname, tel, telegram, policy}: IRegistrationData) {
    if (!policy) throw ApiError.BadRequest(`Отсутствует соглашение с политикой конфиденциальности`)
    const [[candidate]] = await mysql.db.query<{ name: string }>('SELECT name FROM users WHERE email = ?', [email])
    if (candidate) throw ApiError.BadRequest(`Пользователь с почтовым адресом ${email} уже существует`)
    const hashPassword = bcrypt.hashSync(password, 8)
    const activationLink = nanoid()

    const [{insertId: userId}] = await mysql.db.query('INSERT INTO `users` (`email`, `password`, `name`, `surname`, `tel`, `telegram`, `date`) VALUES (?, ?, ?, ?, ?, ?, ?);',
      [email, hashPassword, name, surname, tel, telegram, Date.now()])
    await mysql.db.query('INSERT INTO `activationLinks` (`userId`, `activationLink`) VALUES (?, ?);', [userId, activationLink])
    await mailService.sendMail(email, 'activation', `${process.env.CLIENT_URL}/api/activate/${activationLink}`)

    const userDto = UserDto({id: userId, email, name, surname, tel, telegram, isActivated: false, isPassQuestionnaire: false, status: 2, avatar: undefined})
    const tokens = tokenService.generateTokens(userDto)
    await tokenService.saveToken(userId, tokens.refreshToken)

    setTimeout(async () => {
      const [[user]] = await mysql.db.query<{ isActivated: boolean }>('SELECT `isActivated` FROM `users` WHERE `id` = ?', [userId])
      if (!user?.isActivated) await mysql.db.query('DELETE FROM `users` WHERE (`id` = ?);', [userId])
    }, 900000)

    return {
      refreshToken: tokens.refreshToken,
      userData: {
        user: userDto,
        accessToken: tokens.accessToken,
      },
    }
  }

  async activate(activationLink) {
    const [[link]] = await mysql.db.query<{ userId: number }>('SELECT `userId` FROM `activationLinks` WHERE activationLink = ?', [activationLink])
    if (!link) throw ApiError.BadRequest('Некорректная ссылка активации')
    await mysql.db.query('UPDATE `users` SET `isActivated` = 1 WHERE (`id` = ?);', [link.userId])
    await mysql.db.query('UPDATE `users` SET `status` = 100 WHERE (`id` = ?);', [link.userId])
    await mysql.db.query('DELETE FROM `activationLinks` WHERE (`userId` = ?)', [link.userId])
  }

  async login(email, password) {
    const [[user]] = await mysql.db.query<IUserDto & ILoginData>('SELECT * FROM `users` WHERE (`email` = ?)', [email])
    if (!user) throw ApiError.BadRequest('Пользователь с данным email не был найден')
    const isPassEquals = bcrypt.compareSync(password, user.password)
    if (!isPassEquals) throw ApiError.BadRequest('Неверный пароль')

    user.isPassQuestionnaire = !!(await questionnaireService.getAnswersCount(user.id))
    user.avatar = await avatarService.get(user.id)
    const userDto = UserDto(user)
    const tokens = tokenService.generateTokens(userDto)
    await tokenService.saveToken(userDto.id!, tokens.refreshToken)

    return {
      refreshToken: tokens.refreshToken,
      userData: {
        user: userDto,
        accessToken: tokens.accessToken,
      },
    }
  }

  async logout(refreshToken) {await tokenService.removeToken(refreshToken)}

  async refresh(refreshToken) {
    if (!refreshToken) throw ApiError.Unauthorized()
    const userData = tokenService.validateRefreshToken<IUserDto>(refreshToken)
    const tokenFromDB = await tokenService.findToken(refreshToken)
    if (!userData || !tokenFromDB) throw ApiError.Unauthorized()

    const [[user]] = await mysql.db.query<IUserDto & ILoginData>('SELECT * FROM `users` WHERE (`id` = ?)', [userData.id])
    if (!user) throw ApiError.Conflict('Пользователь не найден')
    user.avatar = await avatarService.get(user.id)
    user.isPassQuestionnaire = !!(await questionnaireService.getAnswersCount(user.id))
    const userDto = UserDto(user)
    const tokens = tokenService.generateTokens(userDto)
    await tokenService.saveToken(userDto.id!, tokens.refreshToken)

    return {
      newRefreshToken: tokens.refreshToken,
      userData: {
        user: userDto,
        accessToken: tokens.accessToken,
      },
    }
  }

  async getStudents() {
    const [users] = await mysql.db.query<IUserDto, '!'>('SELECT `u`.`id`, `u`.`email`, `u`.`name`, `u`.`surname`, `u`.`telegram`, `u`.`status`, `u`.`date`, `a`.`filename` as `avatar` FROM `users` `u` LEFT OUTER JOIN `avatars` `a` ON `u`.`id` = `a`.`userId` WHERE `u`.`status` < 300;')
    return users
  }

  async getStatuses() {
    const [unparsedStatuses] = await mysql.db.query<{ id: number, description: string }, '!'>('select * from statuses')
    let statuses = {}
    for (const {id, description} of unparsedStatuses) statuses[id] = description
    return statuses
  }

  async changeStatuses(statusChanges: IStatusChanges) {
    for (const [userId, {email, status}] of Object.entries(statusChanges)) {
      await mysql.db.query('UPDATE `users` SET `status` = ? WHERE `id` = ? AND `status` < 300;', [status, userId])
      if (status === 200) await mailService.sendMail(email, 'acceptance')
      if (status === 1) await mailService.sendMail(email, 'rejection')
    }
    return await this.getStudents()
  }

  async changeTelegram(telegram: string, userId: number) {
    return await mysql.db.query('UPDATE `users` SET `telegram` = ? WHERE (`id` = ?);', [telegram, userId])
  }
}

export default new userService()
